@@ -1,23 +1,32 @@
 /*
-**  Copyright (c) 2006-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2006-2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-config.h,v 1.40 2007/12/13 23:07:22 msk Exp $
+**  $Id: dkim-config.h,v 1.51 2008/03/04 22:50:22 msk Exp $
 */
 
 #ifndef _DKIM_CONFIG_H_
 #define _DKIM_CONFIG_H_
 
 #ifndef lint
-static char dkim_config_h_id[] = "@(#)$Id: dkim-config.h,v 1.40 2007/12/13 23:07:22 msk Exp $";
+static char dkim_config_h_id[] = "@(#)$Id: dkim-config.h,v 1.51 2008/03/04 22:50:22 msk Exp $";
 #endif /* !lint */
 
 struct configdef dkimf_config[] =
 {
+	{ "AlwaysAddARHeader",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "AlwaysSignHeaders",		CONFIG_TYPE_STRING,	FALSE },
+#ifdef _FFR_AUTHSERV_JOBID
+	{ "AuthservIDWithJobID",	CONFIG_TYPE_BOOLEAN,	FALSE },
+#endif /* _FFR_AUTHSERV_JOBID */
 	{ "AutoRestart",		CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "AutoRestartCount",		CONFIG_TYPE_INTEGER,	FALSE },
+	{ "AutoRestartRate",		CONFIG_TYPE_STRING,	FALSE },
 	{ "Background",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "BodyLengths",		CONFIG_TYPE_BOOLEAN,	FALSE },
+#if _FFR_BODYLENGTH_DB
+	{ "BodyLengthDBFile",		CONFIG_TYPE_STRING,	FALSE },
+#endif /* _FFR_BODYLENGTH_DB */
 	{ "Canonicalization",		CONFIG_TYPE_STRING,	FALSE },
 	{ "ClockDrift",			CONFIG_TYPE_INTEGER,	FALSE },
 	{ "Diagnostics",		CONFIG_TYPE_BOOLEAN,	FALSE },
@@ -27,12 +36,15 @@ struct configdef dkimf_config[] =
 	{ "DNSTimeout",			CONFIG_TYPE_INTEGER,	FALSE },
 	{ "Domain",			CONFIG_TYPE_STRING,	FALSE },
 	{ "ExternalIgnoreList",		CONFIG_TYPE_STRING,	FALSE },
+	{ "FixCRLF",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "KeyFile",			CONFIG_TYPE_STRING,	FALSE },
 	{ "KeyList",			CONFIG_TYPE_STRING,	FALSE },
+	{ "Include",			CONFIG_TYPE_INCLUDE,	FALSE },
 	{ "InternalHosts",		CONFIG_TYPE_STRING,	FALSE },
 	{ "MaximumHeaders",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "MaximumSignedBytes",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "MacroList",			CONFIG_TYPE_STRING,	FALSE },
+	{ "MilterDebug",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "Minimum",			CONFIG_TYPE_STRING,	FALSE },
 	{ "Mode",			CONFIG_TYPE_STRING,	FALSE },
 	{ "MTA",			CONFIG_TYPE_STRING,	FALSE },
@@ -78,10 +90,11 @@ struct configdef dkimf_config[] =
 	{ "StrictTestMode",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "SubDomains",			CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "Syslog",			CONFIG_TYPE_BOOLEAN,	FALSE },
+	{ "SyslogFacility",		CONFIG_TYPE_STRING,	FALSE },
 	{ "SyslogSuccess",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "UMask",			CONFIG_TYPE_INTEGER,	FALSE },
+	{ "UseASPDiscard",		CONFIG_TYPE_BOOLEAN,	FALSE },
 	{ "UserID",			CONFIG_TYPE_STRING,	FALSE },
-	{ "UseSSPDeny",			CONFIG_TYPE_BOOLEAN,	FALSE },
 #ifdef _FFR_VBR
 	{ "VBR-Certifiers",		CONFIG_TYPE_STRING,	FALSE },
 	{ "VBR-TrustedCertifiers",	CONFIG_TYPE_STRING,	FALSE },
