@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.37 2008/01/02 21:20:50 msk Exp $";
+static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.44 2008/03/17 22:03:05 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -380,13 +380,14 @@ static int
 dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
                       int nptrs)
 {
-	u_char *bar;
-	char *ctx;
-	char *colon;
 	int c;
 	int n;
 	int m;
+	int shcnt;
 	size_t len;
+	u_char *bar;
+	char *ctx;
+	char *colon;
 	struct dkim_header *hdr;
 	struct dkim_header **lhdrs;
 	u_char **hdrs;
@@ -403,7 +404,11 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 		for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
 		{
 			if (n >= nptrs)
-				break;
+			{
+				dkim_error(dkim, "too many headers (max %d)",
+				           nptrs);
+				return -1;
+			}
 			ptrs[n] = hdr;
 			n++;
 		}
@@ -435,22 +440,23 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 	lhdrs = DKIM_MALLOC(dkim, n);
 	if (lhdrs == NULL)
 		return -1;
+	memset(lhdrs, '\0', n);
 
-	n = 1;
+	shcnt = 1;
 	for (colon = dkim->dkim_hdrlist; *colon != '\0'; colon++)
 	{
 		if (*colon == ':')
-			n++;
+			shcnt++;
 	}
-	n = dkim->dkim_hdrcnt * n;
+	n = sizeof(u_char *) * shcnt;
 	hdrs = DKIM_MALLOC(dkim, n);
 	if (hdrs == NULL)
 		return -1;
+	memset(hdrs, '\0', n);
 
 	n = 0;
 
 	/* make a split-out copy of hdrlist */
-	memset(hdrs, '\0', sizeof hdrs);
 	for (bar = strtok_r(dkim->dkim_hdrlist, ":", &ctx);
 	     bar != NULL;
 	     bar = strtok_r(NULL, ":", &ctx))
@@ -459,21 +465,11 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 		n++;
 	}
 
-	/* bounds check */
-	if (n > nptrs)
-	{
-		dkim_error(dkim, "too many headers (max %d)", nptrs);
-
-		DKIM_FREE(dkim, lhdrs);
-		DKIM_FREE(dkim, hdrs);
-
-		return -1;
-	}
-
 	/* for each named header, find the last unused one and use it up */
+	shcnt = 0;
 	for (c = 0; c < n; c++)
 	{
-		lhdrs[c] = NULL;
+		lhdrs[shcnt] = NULL;
 
 		len = MIN(DKIM_MAXHEADER, strlen(hdrs[c]));
 		while (len > 0 &&
@@ -488,16 +484,31 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 
 			if (len == hdr->hdr_namelen &&
 			    strncasecmp(hdr->hdr_text, hdrs[c], len) == 0)
-				lhdrs[c] = hdr;
+				lhdrs[shcnt] = hdr;
 		}
 
-		if (lhdrs[c] != NULL)
-			lhdrs[c]->hdr_flags |= DKIM_HDR_SIGNED;
+		if (lhdrs[shcnt] != NULL)
+		{
+			lhdrs[shcnt]->hdr_flags |= DKIM_HDR_SIGNED;
+			shcnt++;
+		}
+	}
+
+	/* bounds check */
+	if (shcnt > nptrs)
+	{
+		dkim_error(dkim, "too many headers (found %d, max %d)", shcnt,
+		           nptrs);
+
+		DKIM_FREE(dkim, lhdrs);
+		DKIM_FREE(dkim, hdrs);
+
+		return -1;
 	}
 
 	/* copy to the caller's buffers */
 	m = 0;
-	for (c = 0; c < n; c++)
+	for (c = 0; c < shcnt; c++)
 	{
 		if (lhdrs[c] != NULL)
 		{
@@ -512,6 +523,70 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 	return m;
 }
 
+/*
+**  DKIM_CANON_FIXCRLF -- rebuffer a body chunk, fixing "naked" CRs and LFs
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	canon -- canonicalization being handled
+**  	buf -- buffer to be fixed
+**  	buflen -- number of bytes at "buf"
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+**
+**  Side effects:
+**  	dkim->dkim_canonbuf will be initialized and used.
+*/
+
+static DKIM_STAT
+dkim_canon_fixcrlf(DKIM *dkim, DKIM_CANON *canon, u_char *buf, size_t buflen)
+{
+	u_char prev;
+	u_char *p;
+	u_char *eob;
+
+	assert(dkim != NULL);
+	assert(canon != NULL);
+	assert(buf != NULL);
+
+	if (dkim->dkim_canonbuf == NULL)
+	{
+		dkim->dkim_canonbuf = dkim_dstring_new(dkim, buflen, 0);
+		if (dkim->dkim_canonbuf == NULL)
+			return DKIM_STAT_NORESOURCE;
+	}
+	else
+	{
+		dkim_dstring_blank(dkim->dkim_canonbuf);
+	}
+
+	eob = buf + buflen - 1;
+
+	prev = canon->canon_lastchar;
+
+	for (p = buf; p <= eob; p++)
+	{
+		if (*p == '\n' && prev != '\r')
+		{
+			dkim_dstring_catn(dkim->dkim_canonbuf, CRLF, 2);
+		}
+		else if (*p == '\r' && p < eob && *(p + 1) != '\n')
+		{
+			dkim_dstring_catn(dkim->dkim_canonbuf, CRLF, 2);
+		}
+		else
+		{
+			dkim_dstring_cat1(dkim->dkim_canonbuf, *p);
+		}
+
+		prev = *p;
+	}
+
+	return DKIM_STAT_OK;
+}
+
+
 /* ========================= PUBLIC SECTION ========================= */
 
 /*
@@ -692,25 +767,23 @@ dkim_add_canon(DKIM *dkim, bool hdr, dkim_canon_t canon, int hashtype,
 	assert(hashtype == DKIM_HASHTYPE_SHA1);
 #endif /* DKIM_HASHTYPE_SHA256 */
 
-	for (cur = dkim->dkim_canonhead; cur != NULL; cur = cur->canon_next)
+	if (!hdr)
 	{
-		if (cur->canon_hdr != hdr ||
-		    cur->canon_hashtype != hashtype ||
-		    cur->canon_canon != canon)
-			continue;
+		for (cur = dkim->dkim_canonhead; cur != NULL; cur = cur->canon_next)
+		{
+			if (cur->canon_hdr ||
+			    cur->canon_hashtype != hashtype ||
+			    cur->canon_canon != canon)
+				continue;
 
-		if (hdr && 
-                    ((hdrlist == NULL && cur->canon_hdrlist != NULL) ||
-                     (hdrlist != NULL && cur->canon_hdrlist == NULL) ||
-                     strcasecmp(hdrlist, cur->canon_hdrlist) != 0))
-			continue;
+			if (length != cur->canon_length)
+				continue;
 
-		if (!hdr && length != cur->canon_length)
-			continue;
+			if (cout != NULL)
+				*cout = cur;
 
-		if (cout != NULL)
-			*cout = cur;
-		return DKIM_STAT_OK;
+			return DKIM_STAT_OK;
+		}
 	}
 
 	new = (DKIM_CANON *) dkim_malloc(dkim->dkim_libhandle,
@@ -846,7 +919,7 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 				     hdr = hdr->hdr_next)
 					hdr->hdr_flags &= ~DKIM_HDR_SIGNED;
 
-				memset(hdrset, '\0', sizeof hdrset);
+				memset(hdrset, '\0', n);
 
 				/* do header selection */
 				nhdrs = dkim_canon_selecthdrs(dkim,
@@ -925,7 +998,7 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 				}
 			}
 
-			memset(hdrset, '\0', sizeof hdrset);
+			memset(hdrset, '\0', n);
 
 			/* do header selection */
 			nhdrs = dkim_canon_selecthdrs(dkim,
@@ -998,7 +1071,11 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 
 				if (!dkim_dstring_catn(dkim->dkim_hdrbuf,
 				                       tmpbuf, tmp - tmpbuf))
+				{
+					DKIM_FREE(dkim, hdrset);
+
 					return DKIM_STAT_NORESOURCE;
+				}
 
 				tmp = tmpbuf;
 			}
@@ -1013,7 +1090,11 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 
 			if (!dkim_dstring_catn(dkim->dkim_hdrbuf,
 			                       tmpbuf, tmp - tmpbuf))
+			{
+				DKIM_FREE(dkim, hdrset);
+
 				return DKIM_STAT_NORESOURCE;
+			}
 		}
 
 		/* canonicalize */
@@ -1224,16 +1305,23 @@ dkim_canon_minbody(DKIM *dkim)
 DKIM_STAT
 dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 {
+	bool fixcrlf;
+	DKIM_STAT status;
 	u_int wlen;
 	DKIM_CANON *cur;
+	size_t plen;
 	u_char *p;
 	u_char *wrote;
 	u_char *eob;
+	u_char *start;
 
 	assert(dkim != NULL);
 
 	dkim->dkim_bodylen += buflen;
 
+	fixcrlf = (dkim->dkim_libhandle->dkiml_flags & DKIM_LIBFLAGS_FIXCRLF) != 0 &&
+	          (dkim->dkim_mode == DKIM_MODE_SIGN);
+
 	for (cur = dkim->dkim_canonhead; cur != NULL; cur = cur->canon_next)
 	{
 		/* skip done hashes and those which are of the wrong type */
@@ -1244,14 +1332,27 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 		if (cur->canon_remain == 0)
 			continue;
 
-		eob = buf + buflen - 1;
-		wrote = buf;
+		start = buf;
+		plen = buflen;
+
+		if (fixcrlf)
+		{
+			status = dkim_canon_fixcrlf(dkim, cur, buf, buflen);
+			if (status != DKIM_STAT_OK)
+				return status;
+
+			start = dkim_dstring_get(dkim->dkim_canonbuf);
+			plen = dkim_dstring_len(dkim->dkim_canonbuf);
+		}
+
+		eob = start + plen - 1;
+		wrote = start;
 		wlen = 0;
 
 		switch (cur->canon_canon)
 		{
 		  case DKIM_CANON_SIMPLE:
-			for (p = buf; p <= eob; p++)
+			for (p = start; p <= eob; p++)
 			{
 				if (*p == '\n')
 				{
@@ -1281,6 +1382,15 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 				}
 				else
 				{
+					if (p == start && fixcrlf &&
+					    cur->canon_lastchar == '\r')
+					{
+						dkim_canon_buffer(cur,
+						                  "\n", 1);
+						cur->canon_lastchar = '\n';
+						cur->canon_blankline = TRUE;
+					}
+
 					if (*p != '\r')
 					{
 						if (cur->canon_blanks > 0)
@@ -1296,7 +1406,7 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 			break;
 
 		  case DKIM_CANON_RELAXED:
-			for (p = buf; p <= eob; p++)
+			for (p = start; p <= eob; p++)
 			{
 				if (*p == '\n')
 				{
@@ -1326,6 +1436,15 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 				}
 				else
 				{
+					if (p == start && fixcrlf &&
+					    cur->canon_lastchar == '\r')
+					{
+						dkim_canon_buffer(cur,
+						                  "\n", 1);
+						cur->canon_lastchar = '\n';
+						cur->canon_blankline = TRUE;
+					}
+
 					/* non-space after a space */
 					if (DKIM_ISLWSP(cur->canon_lastchar) &&
 					    !DKIM_ISLWSP(*p) && *p != '\r')
