@@ -1,10 +1,10 @@
 /*
-**  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifndef lint
-static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.29 2007/10/29 23:14:35 msk Exp $";
+static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.35 2008/03/17 16:40:07 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -154,7 +154,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 
 		if (lib->dkiml_dns_callback == NULL)
 		{
-			status = ar_waitreply(ar, q, NULL, NULL);
+			status = ar_waitreply(ar, q, &anslen, NULL);
 		}
 		else
 		{
@@ -163,7 +163,8 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 				timeout.tv_sec = lib->dkiml_callback_int;
 				timeout.tv_usec = 0;
 
-				status = ar_waitreply(ar, q, NULL, &timeout);
+				status = ar_waitreply(ar, q, &anslen,
+				                      &timeout);
 
 				if (status != AR_STAT_NOREPLY)
 					break;
@@ -178,7 +179,8 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		if (c == 0 && status == AR_STAT_SUCCESS)
 		{
 			memcpy(&hdr, ansbuf, sizeof hdr);
-			if (hdr.tc)
+			if (dkim_check_dns_reply(ansbuf, anslen,
+			                         C_IN, T_TXT) == 2)
 				continue;
 		}
 
@@ -201,7 +203,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 
 	if (lib->dkiml_dns_callback == NULL)
 	{
-		status = ar_waitreply(ar, q, NULL, NULL);
+		status = ar_waitreply(ar, q, &anslen, NULL);
 	}
 	else
 	{
@@ -210,7 +212,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 			timeout.tv_sec = lib->dkiml_callback_int;
 			timeout.tv_usec = 0;
 
-			status = ar_waitreply(ar, q, NULL, &timeout);
+			status = ar_waitreply(ar, q, &anslen, &timeout);
 
 			if (status != AR_STAT_NOREPLY)
 				break;
@@ -256,10 +258,11 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 			return DKIM_STAT_KEYFAIL;
 		}
 	}
+
+	anslen = status;
 #endif /* USE_ARLIB */
 
 	/* set up pointers */
-	anslen = sizeof ansbuf;
 	memcpy(&hdr, ansbuf, sizeof hdr);
 	cp = (u_char *) &ansbuf + HFIXEDSZ;
 	eom = (u_char *) &ansbuf + anslen;
@@ -301,7 +304,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		return DKIM_STAT_NOKEY;
 
 	/* if truncated, we can't do it */
-	if (hdr.tc)
+	if (dkim_check_dns_reply(ansbuf, anslen, C_IN, T_TXT) == 2)
 	{
 		dkim_error(dkim, "`%s' reply truncated", qname);
 		return DKIM_STAT_INTERNAL;
@@ -363,6 +366,13 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 			return DKIM_STAT_INTERNAL;
 		}
 
+		if (ancount > 0)
+		{
+			dkim_error(dkim, "multiple DNS replies for `%s'",
+			           qname);
+			return DKIM_STAT_MULTIDNSREPLY;
+		}
+
 		/* found a record we can use; break */
 		break;
 	}
@@ -480,8 +490,8 @@ dkim_get_key_file(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	snprintf(name, sizeof name, "%s.%s.%s", sig->sig_selector,
 	         DKIM_DNSKEYNAME, sig->sig_domain);
 
-	memset(buf, '\0', sizeof buf);
-	while (fgets(buf, BUFRSZ, f) != NULL)
+	memset(buf, '\0', buflen);
+	while (fgets(buf, buflen, f) != NULL)
 	{
 		p2 = NULL;
 
