@@ -1,10 +1,10 @@
 /*
-**  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifndef lint
-static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.23 2007/11/02 20:15:24 msk Exp $";
+static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.29 2008/03/10 03:48:03 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -171,6 +171,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 	int qtype;
 	int type = -1;
 	int class = -1;
+	int rcnt = 0;
 #ifdef QUERY_CACHE
 	int ttl;
 #endif /* QUERY_CACHE */
@@ -251,7 +252,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 
 		if (lib->dkiml_dns_callback == NULL)
 		{
-			status = ar_waitreply(ar, q, NULL, NULL);
+			status = ar_waitreply(ar, q, &anslen, NULL);
 		}
 		else
 		{
@@ -260,7 +261,8 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 				timeout.tv_sec = lib->dkiml_callback_int;
 				timeout.tv_usec = 0;
 
-				status = ar_waitreply(ar, q, NULL, &timeout);
+				status = ar_waitreply(ar, q, &anslen,
+				                      &timeout);
 
 				if (status != AR_STAT_NOREPLY)
 					break;
@@ -275,7 +277,8 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 		if (c == 0 && status == AR_STAT_SUCCESS)
 		{
 			memcpy(&hdr, ansbuf, sizeof hdr);
-			if (hdr.tc)
+			if (dkim_check_dns_reply(ansbuf, anslen,
+			                         C_IN, T_TXT) == 2)
 				continue;
 		}
 
@@ -298,7 +301,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 
 	if (lib->dkiml_dns_callback == NULL)
 	{
-		status = ar_waitreply(ar, q, NULL, NULL);
+		status = ar_waitreply(ar, q, &anslen, NULL);
 	}
 	else
 	{
@@ -307,7 +310,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 			timeout.tv_sec = lib->dkiml_callback_int;
 			timeout.tv_usec = 0;
 
-			status = ar_waitreply(ar, q, NULL, &timeout);
+			status = ar_waitreply(ar, q, &anslen, &timeout);
 
 			if (status != AR_STAT_NOREPLY)
 				break;
@@ -370,10 +373,11 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 			return 0;
 		}
 	}
+
+	anslen = status;
 #endif /* USE_ARLIB */
 
 	/* set up pointers */
-	anslen = sizeof ansbuf;
 	memcpy(&hdr, ansbuf, sizeof hdr);
 	cp = (u_char *) &ansbuf + HFIXEDSZ;
 	eom = (u_char *) &ansbuf + anslen;
@@ -413,7 +417,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 	}
 
 	/* if truncated, we can't do it */
-	if (hdr.tc)
+	if (dkim_check_dns_reply(ansbuf, anslen, C_IN, T_TXT) == 2)
 	{
 		dkim_error(dkim, "reply for `%s' truncated", query);
 		return -1;
@@ -463,12 +467,19 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
 		}
 		else if (type != qtype)
 		{
-			/* reject anything not valid (stupid wildcards) */
+			/* reject anything not valid (e.g. wildcards) */
 			dkim_error(dkim, "`%s' unexpected reply class/type",
 			           query);
 			return -1;
 		}
 
+		if (!usemx && ancount > 0)
+		{
+			dkim_error(dkim, "multiple DNS replies for `%s'",
+			           query);
+			return DKIM_STAT_MULTIDNSREPLY;
+		}
+
 		/* process it */
 		break;
 	}
