@@ -1,10 +1,10 @@
 /*
-**  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifndef lint
-static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.32 2007/11/28 18:49:49 msk Exp $";
+static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.36 2008/03/13 13:03:06 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -54,7 +54,6 @@ struct nametable *keyflags = prv_keyflags;
 static struct nametable prv_policyparams[] =	/* policy parameters */
 {
 	{ "dkim",	DKIM_PPARAM_POLICY },
-	{ "handling",	DKIM_PPARAM_HANDLING },
 	{ "r",		DKIM_PPARAM_REPORTADDR },
 	{ "t",		DKIM_PPARAM_FLAGS },
 	{ NULL,		-1 }
@@ -65,30 +64,19 @@ static struct nametable prv_policies[] =	/* policies */
 {
 	{ "unknown",	DKIM_POLICY_UNKNOWN },
 	{ "all",	DKIM_POLICY_ALL },
-	{ "strict",	DKIM_POLICY_STRICT },
+	{ "discardable", DKIM_POLICY_DISCARDABLE },
 	{ NULL,		-1 }
 };
 struct nametable *policies = prv_policies;
 
-static struct nametable prv_handlings[] =	/* SSP recommended handling */
-{
-	{ "process",	DKIM_HANDLING_PROCESS },
-	{ "deny",	DKIM_HANDLING_DENY },
-	{ NULL,		-1 }
-};
-struct nametable *handlings = prv_handlings;
-
 static struct nametable prv_policyresults[] =	/* policy results */
 {
 	{ "none",				DKIM_PRESULT_NONE },
+	{ "author domain policy found",		DKIM_PRESULT_AUTHOR },
 	{ "domain does not exist",		DKIM_PRESULT_NXDOMAIN },
-	{ "policy \"unknown\"",			DKIM_PRESULT_UNKNOWN },
-	{ "top-level domain",			DKIM_PRESULT_TOPLEVEL },
-	{ "parent domain check",		DKIM_PRESULT_PARENTOK },
-	{ "policy test flag set",		DKIM_PRESULT_TESTFLAG },
-	{ "policy \"all\" and valid signature",	DKIM_PRESULT_ALLVALIDSIG },
-	{ "valid originator signature",		DKIM_PRESULT_VALIDOSIG },
-	{ "policy check failed",		DKIM_PRESULT_POLICYERROR },
+	{ "parent domain policy found",		DKIM_PRESULT_PARENT },
+	{ "no parent domain policy found",	DKIM_PRESULT_NOPARENT },
+	{ "parent policy doesn't allow subdomains", DKIM_PRESULT_NOSUBDOMAIN },
 	{ NULL,					-1 }
 };
 struct nametable *policyresults = prv_policyresults;
@@ -172,6 +160,7 @@ static struct nametable prv_results[] =		/* result codes */
 	{ "Reject requested",		DKIM_STAT_CBREJECT },
 	{ "Invalid result",		DKIM_STAT_CBINVALID },
 	{ "Try again later",		DKIM_STAT_CBTRYAGAIN },
+	{ "Multiple DNS replies",	DKIM_STAT_MULTIDNSREPLY },
 	{ NULL,				-1 },
 };
 struct nametable *results = prv_results;
@@ -217,6 +206,7 @@ static struct nametable prv_sigerrors[] =	/* signature parsing errors */
 	{ "body hash corrupt",			DKIM_SIGERROR_CORRUPT_BH },
 	{ "signature verification failed",	DKIM_SIGERROR_BADSIG },
 	{ "unauthorized subdomain",		DKIM_SIGERROR_SUBDOMAIN },
+	{ "multiple keys found",		DKIM_SIGERROR_MULTIREPLY },
 	{ NULL,					-1 },
 };
 struct nametable *sigerrors = prv_sigerrors;
