@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
@@ -7,16 +7,16 @@
 #define _DKIM_TEST_H_
 
 #ifndef lint
-static char dkim_test_h_id[] = "@(#)$Id: dkim-test.h,v 1.5 2007/10/17 23:46:51 msk Exp $";
+static char dkim_test_h_id[] = "@(#)$Id: dkim-test.h,v 1.6 2008/02/26 00:04:31 msk Exp $";
 #endif /* !lint */
 
 /* libdkim includes */
 #include "dkim.h"
 
 /* prototypes */
+extern int dkim_test_asp __P((DKIM_LIB *, char *, dkim_policy_t *,
+                              int *, char *, size_t));
 extern int dkim_test_key __P((DKIM_LIB *, char *, char *, char *, size_t,
                               char *, size_t));
-extern int dkim_test_ssp __P((DKIM_LIB *, char *, dkim_policy_t *,
-                              dkim_handling_t *, int *, char *, size_t));
 
 #endif /* ! _DKIM_TEST_H_ */
