@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.160 2008/01/24 01:54:28 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.171 2008/03/23 07:38:38 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -31,7 +31,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.160 2008/01/24 01:54:28 msk Exp $
 **  	pp == patch number
 */
 
-#define	DKIM_LIB_VERSION	0x02040400
+#define	DKIM_LIB_VERSION	0x02050200
 
 /* the basics */
 #ifndef NULL
@@ -99,7 +99,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.160 2008/01/24 01:54:28 msk Exp $
 /* special DNS tokens */
 #define	DKIM_DNSKEYNAME		"_domainkey"
 					/* reserved DNS sub-zone */
-#define	DKIM_DNSPOLICYNAME	"_ssp"	/* reserved DNS sub-zone */
+#define	DKIM_DNSPOLICYNAME	"_asp"	/* reserved DNS sub-zone */
 
 /*
 **  DKIM_STAT -- status code type
@@ -123,6 +123,7 @@ typedef int DKIM_STAT;
 #define	DKIM_STAT_CBINVALID	13	/* callback gave invalid result */
 #define	DKIM_STAT_CBTRYAGAIN	14	/* callback says try again later */
 #define	DKIM_STAT_CBERROR	15	/* callback error */
+#define	DKIM_STAT_MULTIDNSREPLY	16	/* multiple DNS replies */
 
 /*
 **  DKIM_CBSTAT -- callback status code type
@@ -173,6 +174,7 @@ typedef int DKIM_SIGERROR;
 #define DKIM_SIGERROR_CORRUPT_BH	27	/* bh= corrupt */
 #define DKIM_SIGERROR_BADSIG		28	/* signature mismatch */
 #define DKIM_SIGERROR_SUBDOMAIN		29	/* unauthorized subdomain */
+#define DKIM_SIGERROR_MULTIREPLY	30	/* multiple records returned */
 
 /*
 **  DKIM_CANON -- canonicalization method
@@ -261,7 +263,6 @@ typedef int dkim_pparam_t;
 #define DKIM_PPARAM_POLICY	0	/* dkim */
 #define DKIM_PPARAM_FLAGS	1	/* t */
 #define DKIM_PPARAM_REPORTADDR	2	/* r */
-#define DKIM_PPARAM_HANDLING	3	/* handling */
 
 /*
 **  DKIM_POLICY -- policies
@@ -272,23 +273,11 @@ typedef int dkim_policy_t;
 #define DKIM_POLICY_NONE	(-1)	/* none/undefined */
 #define DKIM_POLICY_UNKNOWN	0	/* unknown */
 #define DKIM_POLICY_ALL		1	/* all */
-#define DKIM_POLICY_STRICT	2	/* strict */
+#define DKIM_POLICY_DISCARDABLE	2	/* discardable */
 
 #define DKIM_POLICY_DEFAULT	DKIM_POLICY_UNKNOWN
 #define DKIM_POLICY_DEFAULTTXT	"dkim=unknown"
 
-/*
-**  DKIM_HANDLING -- suspicious message handling
-*/
-
-typedef int dkim_handling_t;
-
-#define	DKIM_HANDLING_NONE	(-1)	/* none/undefined */
-#define	DKIM_HANDLING_PROCESS	0	/* process message */
-#define	DKIM_HANDLING_DENY	1	/* block/reject message */
-
-#define	DKIM_HANDLING_DEFAULT	DKIM_HANDLING_PROCESS
-
 /*
 **  DKIM_PFLAG -- policy flags
 */
@@ -304,14 +293,10 @@ typedef int dkim_handling_t;
 
 #define DKIM_PRESULT_NONE		(-1)	/* none/undefined */
 #define DKIM_PRESULT_NXDOMAIN		0	/* domain does not exist */
-#define DKIM_PRESULT_UNKNOWN		1	/* policy is "unknown" */
-#define DKIM_PRESULT_TOPLEVEL		2	/* top-level domain */
-#define DKIM_PRESULT_PARENTOK		3	/* parent domain check OK */
-#define DKIM_PRESULT_NOSUBDOMAIN	4	/* no subdomains allowed */
-#define DKIM_PRESULT_TESTFLAG		5	/* policy marked "test" */
-#define DKIM_PRESULT_ALLVALIDSIG	6	/* policy "all", valid sig. */
-#define DKIM_PRESULT_VALIDOSIG		7	/* valid originator sig. */
-#define DKIM_PRESULT_POLICYERROR	8	/* policy check failed */
+#define DKIM_PRESULT_AUTHOR		1	/* ASP found for author */
+#define DKIM_PRESULT_NOPARENT		2	/* no parent ASP found */
+#define DKIM_PRESULT_NOSUBDOMAIN	3	/* parent ASP disallows subs */
+#define DKIM_PRESULT_PARENT		4	/* ASP found for parent */
 
 /*
 **  DKIM_MODE -- mode of a handle
@@ -377,15 +362,16 @@ typedef int dkim_opts_t;
 #define	DKIM_OPTS_SIGNATURETTL	11
 #define	DKIM_OPTS_CLOCKDRIFT	12
 
-#define	DKIM_LIBFLAGS_NONE		0x00
-#define	DKIM_LIBFLAGS_TMPFILES		0x01
-#define	DKIM_LIBFLAGS_KEEPFILES		0x02
-#define	DKIM_LIBFLAGS_SIGNLEN		0x04
-#define DKIM_LIBFLAGS_CACHE		0x08
-#define DKIM_LIBFLAGS_ZTAGS		0x10
-#define DKIM_LIBFLAGS_DELAYSIGPROC	0x20
-#define DKIM_LIBFLAGS_EOHCHECK		0x40
-#define DKIM_LIBFLAGS_ACCEPTV05		0x80
+#define	DKIM_LIBFLAGS_NONE		0x000
+#define	DKIM_LIBFLAGS_TMPFILES		0x001
+#define	DKIM_LIBFLAGS_KEEPFILES		0x002
+#define	DKIM_LIBFLAGS_SIGNLEN		0x004
+#define DKIM_LIBFLAGS_CACHE		0x008
+#define DKIM_LIBFLAGS_ZTAGS		0x010
+#define DKIM_LIBFLAGS_DELAYSIGPROC	0x020
+#define DKIM_LIBFLAGS_EOHCHECK		0x040
+#define DKIM_LIBFLAGS_ACCEPTV05		0x080
+#define DKIM_LIBFLAGS_FIXCRLF		0x100
 
 #define	DKIM_LIBFLAGS_DEFAULT		DKIM_LIBFLAGS_NONE
 
@@ -612,6 +598,20 @@ extern DKIM_STAT dkim_eoh __P((DKIM *dkim));
 
 extern DKIM_STAT dkim_body __P((DKIM *dkim, u_char *buf, size_t len));
 
+/*
+**  DKIM_CHUNK -- process a message chunk
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	buf -- data to process
+**  	buflen -- number of bytes at "buf" to process
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_chunk __P((DKIM *dkim, u_char *buf, size_t buflen));
+
 /*
 **  DKIM_EOM -- identify end of body
 **
@@ -627,6 +627,49 @@ extern DKIM_STAT dkim_body __P((DKIM *dkim, u_char *buf, size_t len));
 
 extern DKIM_STAT dkim_eom __P((DKIM *dkim, bool *testkey));
 
+/*
+**  DKIM_KEY_SYNTAX -- process a key record parameter set for valid syntax
+**
+**  Parameters:
+**  	dkim -- DKIM context in which this is performed
+**  	str -- string to be scanned
+**  	len -- number of bytes available at "str"
+**
+**  Return value:
+**  	A DKIM_STAT constant.
+*/
+
+extern DKIM_STAT dkim_key_syntax __P((DKIM *dkim, u_char *str, size_t len));
+
+/*
+**  DKIM_POLICY_SYNTAX -- process a policy record parameter set
+**                        for valid syntax
+**
+**  Parameters:
+**  	dkim -- DKIM context in which this is performed
+**  	str -- string to be scanned
+**  	len -- number of bytes available at "str"
+**
+**  Return value:
+**  	A DKIM_STAT constant.
+*/
+
+extern DKIM_STAT dkim_policy_syntax __P((DKIM *dkim, u_char *str, size_t len));
+
+/*
+**  DKIM_SIG_SYNTAX -- process a signature parameter set for valid syntax
+**
+**  Parameters:
+**  	dkim -- DKIM context in which this is performed
+**  	str -- string to be scanned
+**  	len -- number of bytes available at "str"
+**
+**  Return value:
+**  	A DKIM_STAT constant.
+*/
+
+extern DKIM_STAT dkim_sig_syntax __P((DKIM *dkim, u_char *str, size_t len));
+
 /*
 **  DKIM_GETCACHESTATS -- retrieve cache statistics
 **
@@ -782,7 +825,7 @@ extern DKIM_STAT dkim_reportinfo __P((DKIM *dkim, DKIM_SIGINFO *sig,
 **  DKIM_OPTIONS -- set/get options
 **
 **  Parameters:
-**  	dklib -- DKIM library handle
+**  	dkimlib -- DKIM library handle
 **  	op -- operation (DKIM_OPT_GET or DKIM_OPT_SET)
 **  	opt -- which option (a DKIM_OPTS_* constant)
 **  	ptr -- value (in or out)
@@ -792,7 +835,7 @@ extern DKIM_STAT dkim_reportinfo __P((DKIM *dkim, DKIM_SIGINFO *sig,
 **  	A DKIM_STAT value.
 */
 
-extern DKIM_STAT dkim_options __P((DKIM_LIB *dklib, int op, dkim_opts_t opt,
+extern DKIM_STAT dkim_options __P((DKIM_LIB *dkimlib, int op, dkim_opts_t opt,
                                    void *ptr, size_t len));
 
 /*
@@ -1165,19 +1208,14 @@ extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 **
 **  Parameters:
 **  	dkim -- DKIM handle
-**  	test -- policy test flag (returned)
-**  	susp -- suspicious message flag (returned)
 **  	pcode -- discovered policy (returned)
-**  	hcode -- recommended suspicious message handling (returned)
 **  	pstate -- state, for re-entrancy (updated; can be NULL)
 **
 **  Return value:
 **  	A DKIM_STAT_* constant.
 */
 
-extern DKIM_STAT dkim_policy __P((DKIM *dkim, bool *test, bool *susp,
-                                  dkim_policy_t *pcode,
-                                  dkim_handling_t *hcode,
+extern DKIM_STAT dkim_policy __P((DKIM *dkim, dkim_policy_t *pcode,
                                   DKIM_PSTATE *pstate));
 
 /*
@@ -1255,18 +1293,6 @@ extern int dkim_getpresult __P((DKIM *dkim));
 
 extern const char *dkim_getpresultstr __P((int presult));
 
-/*
-**  DKIM_GETHANDLINGSTR -- retrieve handling result string
-**
-**  Parameters:
-**  	hresult -- policy result code to translate
-**
-**  Return value:
-**  	Pointer to text that describes "hresult".
-*/
-
-extern const char *dkim_gethandlingstr __P((int hresult));
-
 /*
 **  DKIM_GETPOLICYSTR -- retrieve sender policy string
 **
@@ -1320,6 +1346,34 @@ extern DKIM_STAT dkim_diffheaders __P((DKIM *dkim, int maxcost, char **ohdrs,
                                        int *nout));
 #endif /* _FFR_DIFFHEADERS */
 
+#ifdef _FFR_BODYLENGTH_DB
+/*
+**  DKIM_GETPARTIAL -- return a DKIM handle's "body length tag" flag
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	True iff the signature is to include a body length tag
+*/
+
+extern bool dkim_getpartial __P((DKIM *dkim));
+
+/*
+**  DKIM_SETPARTIAL -- set the DKIM handle to sign using the DKIM body length
+**                     tag (l=)
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	value -- new flag value
+**
+**  Return value:
+**  	DKIM_STAT_OK
+*/
+
+extern DKIM_STAT dkim_setpartial __P((DKIM *dkim, bool value));
+#endif /* _FFR_BODYLENGTH_DB */
+
 /*
 **  RFC2822_MAILBOX_SPLIT -- extract the userid and host from a structured
 **                           header
