@@ -1,10 +1,10 @@
 /*
-**  Copyright (c) 2005, 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005, 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifndef lint
-static char rfc2822_c_id[] = "@(#)$Id: rfc2822.c,v 1.3 2007/11/12 22:08:18 msk Exp $";
+static char rfc2822_c_id[] = "@(#)$Id: rfc2822.c,v 1.4 2008/01/09 21:19:53 msk Exp $";
 #endif /* !lint */
 
 /* system inludes */
@@ -26,7 +26,7 @@ typedef unsigned long cmap_elem_type;
 #define	CMAP_NBITS	 	(sizeof(cmap_elem_type) * CHAR_BIT)
 #define	CMAP_NELEMS	  	((1 + UCHAR_MAX) / CMAP_NBITS)
 #define	CMAP_INDEX(i)		((unsigned char)(i) / CMAP_NBITS)
-#define	CMAP_BIT(i)  		(1 << (unsigned char)(i) % CMAP_NBITS)
+#define	CMAP_BIT(i)  		(1L << (unsigned char)(i) % CMAP_NBITS)
 #define	CMAP_TST(ar, c)    	((ar)[CMAP_INDEX(c)] &  CMAP_BIT(c))
 #define	CMAP_SET(ar, c)    	((ar)[CMAP_INDEX(c)] |= CMAP_BIT(c))
 
