@@ -1,10 +1,10 @@
 /*
-**  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifndef lint
-static char t_setup_c_id[] = "@(#)$Id: t-setup.c,v 1.6 2007/09/17 16:37:22 msk Exp $";
+static char t_setup_c_id[] = "@(#)$Id: t-setup.c,v 1.7 2008/01/15 20:58:46 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -53,6 +53,14 @@ main(int argc, char **argv)
 	}
 	fprintf(f, "\n");
 
+	fprintf(f, "%s.%s.%s ", SELECTOR, DKIM_DNSKEYNAME, DOMAIN2);
+	for (p = PUBLICKEYNOS; *p != '\0'; p++)
+	{
+		if (*p != '\n')
+			putc(*p, f);
+	}
+	fprintf(f, "\n");
+
 	fprintf(f, "%s.%s.%s ", DKIM_DNSPOLICYNAME, DKIM_DNSKEYNAME, DOMAIN2);
 	fprintf(f, "dkim=all; r=%s\n", REPLYADDRESS);
 
