@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test49_c_id[] = "@(#)$Id: t-test49.c,v 1.8 2007/08/02 06:08:42 msk Exp $";
+static char t_test49_c_id[] = "@(#)$Id: t-test49.c,v 1.9 2007/10/24 06:25:57 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -15,9 +15,9 @@ static char t_test49_c_id[] = "@(#)$Id: t-test49.c,v 1.8 2007/08/02 06:08:42 msk
 #include <unistd.h>
 
 /* libdb includes */
-#ifdef _FFR_QUERY_CACHE
+#ifdef QUERY_CACHE
 # include <db.h>
-#endif /* _FFR_QUERY_CACHE */
+#endif /* QUERY_CACHE */
 
 /* libdkim includes */
 #include "dkim.h"
@@ -41,10 +41,10 @@ static char t_test49_c_id[] = "@(#)$Id: t-test49.c,v 1.8 2007/08/02 06:08:42 msk
 int
 main(int argc, char **argv)
 {
-#ifndef _FFR_QUERY_CACHE
+#ifndef QUERY_CACHE
 	printf("*** query caching SKIPPED\n");
 
-#else /* ! _FFR_QUERY_CACHE */
+#else /* ! QUERY_CACHE */
 
 	int status;
 	int err;
@@ -88,7 +88,7 @@ main(int argc, char **argv)
 	assert(status == 1);
 
 	dkim_cache_close(cache);
-#endif /* ! _FFR_QUERY_CACHE */
+#endif /* ! QUERY_CACHE */
 
 	return 0;
 }
