@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.16 2008/01/02 21:20:50 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.20 2008/02/07 02:14:41 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -13,6 +13,11 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.16 2008/01/02 21:20:50 msk Exp $"
 #include <assert.h>
 #include <string.h>
 #include <errno.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <arpa/nameser.h>
+#include <netdb.h>
+#include <resolv.h>
 
 /* libsm includes */
 #include <sm/string.h>
@@ -21,6 +26,12 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.16 2008/01/02 21:20:50 msk Exp $"
 #include "dkim.h"
 #include "util.h"
 
+#if defined(__RES) && (__RES >= 19940415)
+# define RES_UNC_T		char *
+#else /* __RES && __RES >= 19940415 */
+# define RES_UNC_T		unsigned char *
+#endif /* __RES && __RES >= 19940415 */
+
 /*
 **  DKIM_COLLAPSE -- remove spaces from a string
 **
@@ -607,3 +618,144 @@ dkim_strtoull(const char *str, char **endptr, int base)
 	else
 		return value;
 }
+
+/*
+**  DKIM_CHECK_DNS_REPLY -- see if a DNS reply is truncated or corrupt
+**
+**  Parameters:
+**  	ansbuf -- answer buffer
+**  	anslen -- number of bytes returned
+**  	xclass -- expected class
+**  	xtype -- expected type
+**
+**  Return value:
+**  	2 -- reply not usable
+**  	1 -- reply truncated but usable
+**  	0 -- reply intact (but may not be what you want)
+**  	-1 -- other error
+*/
+
+int
+dkim_check_dns_reply(unsigned char *ansbuf, size_t anslen,
+                     int xclass, int xtype)
+{
+	bool trunc = FALSE;
+	int qdcount;
+	int ancount;
+	int n;
+	uint16_t type;
+	uint16_t class;
+	uint32_t ttl;
+	unsigned char *cp;
+	unsigned char *eom;
+	HEADER hdr;
+	unsigned char name[MAXHOSTNAMELEN + 1];
+
+	assert(ansbuf != NULL);
+
+	/* set up pointers */
+	memcpy(&hdr, ansbuf, sizeof hdr);
+	cp = ansbuf + HFIXEDSZ;
+	eom = ansbuf + anslen;
+
+	/* skip over the name at the front of the answer */
+	for (qdcount = ntohs((unsigned short) hdr.qdcount);
+	     qdcount > 0;
+	     qdcount--)
+	{
+		/* copy it first */
+		(void) dn_expand((unsigned char *) ansbuf, eom, cp, name,
+		                 sizeof name);
+
+		if ((n = dn_skipname(cp, eom)) < 0)
+			return 2;
+
+		cp += n;
+
+		/* extract the type and class */
+		if (cp + INT16SZ + INT16SZ > eom)
+			return 2;
+
+		GETSHORT(type, cp);
+		GETSHORT(class, cp);
+	}
+
+	if (type != xtype || class != xclass)
+		return 0;
+
+	/* if NXDOMAIN, return DKIM_STAT_NOKEY */
+	if (hdr.rcode == NXDOMAIN)
+		return 0;
+
+	/* if truncated, we can't do it */
+	if (hdr.tc)
+		trunc = TRUE;
+
+	/* get the answer count */
+	ancount = ntohs((unsigned short) hdr.ancount);
+	if (ancount == 0)
+		return (trunc ? 2 : 0);
+
+	/*
+	**  Extract the data from the first TXT answer.
+	*/
+
+	while (--ancount >= 0 && cp < eom)
+	{
+		/* grab the label, even though we know what we asked... */
+		if ((n = dn_expand((unsigned char *) ansbuf, eom, cp,
+		                   (RES_UNC_T) name, sizeof name)) < 0)
+			return 2;
+
+		/* ...and move past it */
+		cp += n;
+
+		/* extract the type and class */
+		if (cp + INT16SZ + INT16SZ + INT32SZ > eom)
+			return 2;
+
+		GETSHORT(type, cp);
+		GETSHORT(class, cp);
+		GETLONG(ttl, cp);
+
+		/* skip CNAME if found; assume it was resolved */
+		if (type == T_CNAME)
+		{
+			char chost[DKIM_MAXHOSTNAMELEN + 1];
+
+			if ((n = dn_expand((u_char *) ansbuf, eom, cp,
+			                   name, sizeof name)) < 0)
+				return 2;
+
+			cp += n;
+			continue;
+		}
+		else if (type != xtype)
+		{
+			return (trunc ? 1 : 0);
+		}
+
+		/* found a record we can use; break */
+		break;
+	}
+
+	/* if ancount went below 0, there were no good records */
+	if (ancount < 0)
+		return (trunc ? 1 : 0);
+
+	/* get payload length */
+	if (cp + INT16SZ > eom)
+		return 2;
+
+	GETSHORT(n, cp);
+
+	/*
+	**  XXX -- maybe deal with a partial reply rather than require
+	**  	   it all
+	*/
+
+	if (cp + n > eom)
+		return 2;
+
+	return (trunc ? 1 : 0);
+}
